/*

DESCRIPTION:
    Main include file for 12VDC->5VDC SMPS non-isolated buck based on full-bridge
    inverter development using PIC16F1769.

SYNTAX:  n/a

KNOWN BUGS:  n/a.

COPYRIGHT:  (c) 2017, Brian Cornell, all rights reserved.

DEPENDENCIES:  none

VERSION:  1.2, production release

MODIFICATION HISTORY:

    23 Mar 18, BWC, added conditional compilation code to support 12V output
        with an input at 12V and option to disable output SSR.
    28 Oct 17, BWC, created file.


*/



//#define _DEBUG                              //Uncomment for conditional testing

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Uncomment to disable the error amp (comparator) output for open loop testing.
 * This also disables output voltage limit controls.
 * 
 * IMPORTANT:  _DEBUG & _OPEN_LOOP are mutually exclusive.
 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//#define _OPEN_LOOP

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Uncomment to disable the output voltage limit controls.
 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//#define _NO_OUTPUT_VLIMIT

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * PRG2 falling event timing method:  uncomment to use CLC1 to control PRG2 FE 
 * timing.  This essentially has LS mirror HS duty cycle (minus DAC4 value).
 * This method is efficient at light loads (below critical inductor current) but
 * inefficient at higher loads since LS switch body diode comes into conduction.
 * 
 * If commented out then PWM6 is connected directly to PRG2 FE input;
 * which keeps LS on for duration of period minus DAC4 value and any slewing
 * of PWM6 DC below 50% (has the effect of progressively turning off LS earlier).
 * This approach is more efficient at loads above critical inductor current but
 * below this there's a small loss in efficiency since the LS is left on too
 * long.
 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//#define _USE_PRG2_FE_CLC1

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Uncomment for 12V output.
 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define _OUTPUT_12V


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Uncomment if output SSR not used.
 +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define _DISABLE_OUTPUT_SSR


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                      Version constants & definitions
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define VERMAJOR    1
#define VERMINOR    2
#define VERBUILD    2


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * System wide constants & definitions.  Types:
 *  + DC=Duty Cycle
 *  + EN=ENable bit
 *  + RT=RaTio of a voltage presented to PIC, internal or external
 *  + LM=LiMiting operational value
 *  + TM=TiMing
 *  + MP=Peripheral port bit MaPping
 * 
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#ifdef _OUTPUT_12V
#define S_DC_MAX        0X0097              //Maximum allowable duty cycle
#define S_RT_OUT        6                   //Output voltage ratio
#define S_LM_BSM        11000               //Minimum DC bus voltage
#define S_LM_BSR        11700               //DC bus voltage to allow restart when in UVLO
#define S_LM_OTL        10000               //Minimum allowable output voltage
#define S_LM_OTH        13000               //Maximum allowable output voltage
#else
#define S_DC_MAX        0X004F              //Maximum allowable duty cycle
#define S_RT_OUT        2                   //Output voltage ratio
#define S_LM_BSM        11000               //Minimum DC bus voltage
#define S_LM_BSR        11700               //DC bus voltage to allow restart when in UVLO
#define S_LM_OTL        4800                //Minimum allowable output voltage
#define S_LM_OTH        5200                //Maximum allowable output voltage
#endif

#define S_DC_MIN        0x04                //Minimum allowable duty cycle
#define S_DC_MIN_P      ((S_DC_MIN * 100) / (CFG_PWM_5_FRQ + 1))
#define S_DC_MAX_P      ((S_DC_MAX * 100) / (CFG_PWM_5_FRQ + 1))
#define S_EN_ADC        0x01                //Enable bit position in ADCON0 register
#define S_RT_BUS        5                   //Bus voltage ratio
#define S_LM_BSX        20000               //Maximum DC bus voltage
#define S_LM_LSOT       14000               //Voltage to switch to full LS on-time
#define S_TM_RST        200                 //# mS to wait before reseting PIC
#define S_TM_INT        300                 //# mS for inrush & settle time at power-up
#define S_MP_CSN        C2OUT               //Comparator 2 output bit for over-current sense
#define S_LM_BSM_HYS    200                 //Hysteresis for DC bus voltage limit
#define S_LM_OTL_HYS    1000                //Hysteresis for min output voltage limit
#define S_LM_OTH_HYS    200                 //Hysteresis for max output voltage limit
#define S_TM_STOP       2000                //#mS to wait after shutdown
#define S_LM_OVC        10                  //# of over-current events that can occur in single monitoring loop
#define S_LM_OVC_HYS    200                  //Hysteresis for over-current event limit
#define S_TM_BLK_SPD    650                 //Speed, in mS, to blink error code on status indicator
#define S_TM_INRSHBP    10                  //#mS to wait after enabling inrush bypass


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                         Object sizes & positions
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define SIZ_INT_ADC 2                   //# bytes in ADC integer
#define POS_HIG_ADC 1                   //ADC result high byte
#define POS_LOW_ADC 0                   //ADC result low byte


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Analog channel mapping:  channel=port=pin.  The values assigned to these
 * constants should match those used by ADC.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define ANLCAN0     0x00                //AN0=RA0=19
#define ANLCAN1     0x04                //AN1=RA1=18
#define ANLCAN2     0x08                //AN2=RA2=17
#define ANLCAN3     0x0C                //AN3=RA4=3
#define ANLCAN4     0x10                //AN4=RC0=16
#define ANLCAN5     0x14                //AN5=RC1=15
#define ANLCAN6     0x18                //AN6=RC2=14
#define ANLCAN7     0x1C                //AN7=RC3=7
#define ANLCAN8     0x20                //AN8=RC6=8
#define ANLCAN9     0x24                //AN9=RC7=9
#define ANLCAN10    0x28                //AN10=RB4=13
#define ANLCAN11    0x2C                //AN11=RB5=12


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                           PPS & direct pin mapping
 * 
 * For PPS output pin assignments both aliases for pin registers and constants 
 * mapping what peripheral connects to the pin.
 *      Pin register alias convention:  PPS_P_XXX_YY where
 *          XXX=peripheral name
 *          YY=channel or other detail; can be omitted
 *      Peripheral mapping convention:  PPS_M_XXX_YY same as alias
 * 
 * Input pins only require mapping constant.
 * 
 * For pins accessed directly or assigned in a peripheral's register (like ADC)
 * use the following convention:  PIN_X_YYYYYY where
 *          X=Input or Output
 *          YYYYYY=user assigned mnemonic describing function
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define PPS_P_COG1_A    RB5PPS          //COG1 A output signal on pin 12
#define PPS_M_COG1_A    0x04            //Output COG1 A signal on selected output

#ifdef _DEBUG
#define PPS_P_COG1_B    RC7PPS          //COG1  output signal on pin 9
#define PPS_M_COG1_B    0x05            //Output COG1 B signal on selected output
#endif

#define PPS_P_CLC2_Q    RB6PPS          //CLC2 output signal on pin 11
#define PPS_M_CLC2_Q    0x02            //Output CLC2 signal on selected output
#define PIN_I_DCBUSV    ANLCAN2         //DC bus voltage monitored on pin 17
#define PIN_I_OUTPTV    ANLCAN4         //Output voltage monitored on pin 16

#ifdef _USE_PRG2_FE_CLC1
#define PPS_P_PRG2_FS   PRG2FPPS        //PRG2 falling source input pin selection
#define PPS_M_PRG2_FS   0x0F            //PRG2 pin=RB7
#define PPS_P_CLC1_Q    RB7PPS          //CLC1 output signal on pin 10
#define PPS_M_CLC1_Q    0x01            //Output CLC1 signal on selected output
#endif

#define PIN_O_FAULT     RC4             //Fault indicator pin
#define PIN_O_RUN       RC5             //Run indicator pin
#define PPS_P_PWM_3     RC6PPS          //PWM3 output signal on pin 8
#define PPS_M_PWM_3     0x0E            //Output PWM3 signal on selected output
#define PIN_O_INRUSH    RA5             //Controls inrush bypass relay


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Port/pin configuration:  I/O, analog/digital, slew rate, etc.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//Port I/O:  1=input, 0=output
#define IO_PORTA    0b11011111
#define IO_PORTB    0b00011111
#ifdef _DEBUG
#define IO_PORTC    0b00001011
#else
#ifdef _OPEN_LOOP
#define IO_PORTC    0b10001111
#else
#define IO_PORTC    0b10001011
#endif
#endif

//Port Analog/Digital:  1=analog, 0=digital
#define AD_PORTA    0b11011111
#define AD_PORTB    0b00011111
#ifdef _DEBUG
#define AD_PORTC    0b00001111
#else
#define AD_PORTC    0b10001111
#endif

//Port weak pull-up:  1=enabled, 0=disabled
#define WP_PORTA    0b00000000
#define WP_PORTB    0b00000000
#define WP_PORTC    0b00000000
//Open drain:  1=open drain, 0=push-pull
#define OD_PORTA    0b00000000
#define OD_PORTB    0b00000000
#define OD_PORTC    0b00000000
//Slew rate:  1=limited, 0=maximum
#define SR_PORTA    0b00000000
#define SR_PORTB    0b00000000
#define SR_PORTC    0b00000000
//Input level:  1=ST, 0=TLL
#define IL_PORTA    0b00000000
#define IL_PORTB    0b00000000
#define IL_PORTC    0b00000000


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                            Macro functions
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//Status indicator control
#define setIndFAULT()       PIN_O_FAULT = TRUE; PIN_O_RUN = FALSE
#define setIndRUN()         PIN_O_FAULT = FALSE; PIN_O_RUN = TRUE
#define setIndOFF()         PIN_O_FAULT = FALSE; PIN_O_RUN = FALSE

//Output switch control
#ifndef _DISABLE_OUTPUT_SSR
#define setOutputON()       PWM3CONbits.EN = TRUE
#define setOutputOFF()      PWM3CONbits.EN = FALSE
#else
#define setOutputON()       NOP()
#define setOutputOFF()      NOP()
#endif

//Inrush bypass control
#define setInrushbpON()     PIN_O_INRUSH = TRUE;
#define setInrushbpOFF()    PIN_O_INRUSH = FALSE;


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *              ADC configuration & acquisition constants
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//AD acquisition configuration & control constants
#define ADCCFGC1    0xD3                //Result right justified, Fosc/16, Vss, FVR
#define ADCCFGC2    0x00                //No auto conversion
#define ADCACQTIM   10                  //# uS required for sample & hold cap to acquire
#define ADCMAXVAL   1023                //Maximum value 10-bit ADC can return

//Internal channels (peripherals)
#define ADC_CH_FVR      0x7C            //FVR
#define ADC_CH_DAC_1    0x78            //DAC1
#define ADC_CH_TMP      0x74            //Temperature indicator
#define ADC_CH_DAC_2    0x70            //DAC2
#define ADC_CH_DAC_3    0x6C            //DAC3
#define ADC_CH_DAC_4    0x68            //DAC4


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Control constants for function ReadVoltage
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define FRV_ADC_AQT 20                  //# uS required for sample & hold cap to acquire
#define FRV_ADC_V1V 250                 //ADC value for 1V when Vref=4.096


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Control constants for function chgDutyCycle.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define FCDC_DCDELAY    1000            //# uS to wait between each DC increment


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * FVR, DAC, and function Initialize configuration constants
 * DAC1:  over-current sense
 * DAC2:  error amp & output voltage sense
 * DAC3:  ramp generator (PRG)
 * DAC4:  LS on limit threshold
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_FVR_C       0xCF            //Enabled, no temp sense, CDAFVRx4, ADFVRx4

//DAC1
#define CFG_DAC_1_C     0x88            //Enabled, right justified, FVR buffer 2, Vss
#ifdef _OUTPUT_12V
//1.64V (~9.5A)
#define CFG_DAC_1_VH    0x01            //Output high byte
#define CFG_DAC_1_VL    0x81            //Output low byte
#else
//920mV (~5.5A)
#define CFG_DAC_1_VH    0x00            //Output high byte
#define CFG_DAC_1_VL    0xE6            //Output low byte
#endif

//DAC2
#define CFG_DAC_2_C     0x88            //Enabled, right justified, FVR buffer 2, Vss
#ifdef _OUTPUT_12V
//1.96V
#define CFG_DAC_2_VH    0x01            //Output high byte
#define CFG_DAC_2_VL    0xFE            //Output low byte
#else
//2.5V
#define CFG_DAC_2_VH    0x02            //Output high byte
#define CFG_DAC_2_VL    0x71            //Output low byte
#endif

//DAC3:  0V
#define CFG_DAC_3_C     0x80            //Enabled, Vdd, Vss
#define CFG_DAC_3_V     0x00            //Output byte

//DAC4
#define CFG_DAC_4_C     0x88            //Enabled, FVR buffer 2, Vss
#ifdef _OUTPUT_12V
//0.128V
#define CFG_DAC_4_V     0x01            //Output byte
#else
//0.768V (set based on testing)
#define CFG_DAC_4_V     0x06            //Output byte
#endif

#define FI_FVR_DLY      30              //Time delay in uS for FVR buffer to stabilize


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Error amplifier OPA1 configuration
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#ifndef _OPEN_LOOP
#define CFG_OPA_1_C     0x80            //Enabled, override disabled
#else
#define CFG_OPA_1_C     0x00            //Disabled, override disabled
#endif
#define CFG_OPA_1_N     0x00            //Connect (-) to OPA1IN0- pin
#define CFG_OPA_1_P     0x03            //Connect (+) to DAC2_out


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Configure comparator 1 as falling trigger for COG; (-) from error amp, (+) 
 * from PRG.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_CM_1_C0     0x9E            //Enabled, output inverted, ZLF on, HYS on, no sync
#define CFG_CM_1_C1     0x00            //No interrupts
#define CFG_CM_1_N      0x02            //Connect (-) to C1IN2- pin (14)
#define CFG_CM_1_P      0x08            //Connect (+) to PRG1 output

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Configure comparator 2 for buck switch over-current sense.  Output used by 
 * COG as a falling event input.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_CM_2_C0     0x8E            //Enabled, ZLF on, HYS on, no sync
#define CFG_CM_2_C1     0x01            //Interrupt on falling edge
#define CFG_CM_2_N      0x01            //Connect (-) to C2IN1- pin (15)
#define CFG_CM_2_P      0x0A            //Connect (+) to DAC1 output

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Configure comparator 3 as input to CLC2 for low-side switch control; (-) from
 * PRG2, (+) from DAC4
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_CM_3_C0     0x9E            //Enabled, output inverted, ZLF on, HYS on, no sync
#define CFG_CM_3_C1     0x00            //No interrupts
#define CFG_CM_3_N      0x05            //Connect (-) to PRG2 output
#define CFG_CM_3_P      0x0D            //Connect (+) to DAC4 output


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                          Timer configurations
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//Timer2 to support 10-bit PWMs
#define CFG_TMR_2_CLK   0x01            //Clock source=Fosc/4
#define CFG_TMR_2_CON   0x00            //Off, no pre/post-scale
#define CFG_TMR_2_PR    0x27            //Period (frequency) = 100kHz


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                          PWM configurations
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//Configure 10-bit PWM3:  output switch gate drive
#define CFG_PWM_3_CTRL          0x00    //Disabled, active high output
//#define CFG_PWM_3_DCYCLE        0x14    //Set gate drive voltage ~ 12V
#define CFG_PWM_3_DCYCLE        0x10    //Set gate drive voltage ~ 12V

//Configure 16-bit PWM5:  master and sets max DC
#define CFG_PWM_5_CTRL          0x80    //Enabled, active high output, standard mode
#define CFG_PWM_5_TMR           0x0000  //Starting value for timer
#define CFG_PWM_5_FRQ           0x009f  //Frequency (100kHz)
#define CFG_PWM_5_OFFSET        0x0000  //Offset
#define CFG_PWM_5_PHASE         0x0000  //Phase
#define CFG_PWM_5_LDATRIG       0x80    //Load triggering controlled by LDA bit only
#define CFG_PWM_5_CLOCK         0x01    //No prescale, use HFINTOSC as clock source

//Configure 16-bit PWM6:  slave and sets max ramp-up time for PRG2
#define CFG_PWM_6_CTRL          0x80    //Enabled, active high output, standard mode
#define CFG_PWM_6_CLOCK         0x01    //No prescale, use HFINTOSC as clock source
#define CFG_PWM_6_LDATRIG       0x80    //Load triggering controlled by LDA bit only
#define CFG_PWM_6_TRIGGER       0x60    //Continuous slave run mode with offset triggered timer reset and synchronized start, OF match on PWM5
#define CFG_PWM_6_PHASE         0x0000  //Phase
#define CFG_PWM_6_DCYCLE        ((CFG_PWM_5_FRQ / 2) - 2) //Set max ramp-up time < 50%
#define CFG_PWM_6_FRQ           CFG_PWM_5_FRQ   //Frequency
#define CFG_PWM_6_OFFSET        0x0000  //Offset
#define CFG_PWM_6_TMR           0x0000  //Starting timer count


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Configure Complementary Output Generator 1 (COG1)
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_COG_1_FE_BLANK      0x00    //No falling event blanking
#define CFG_COG_1_AS_CONTROL    0xA8    //Auto-shutdown control:  shutdown, all outputs set to logic '0'
#define CFG_COG_1_RE_ONTIME     0x02    //PWM5 starts on-time
#define CFG_COG_1_RM_ONTIME     0x02    //Mode for PWM5 is after phase delay
#define CFG_COG_1_FE_OVRCRNT    0x04    //Comparator 2 is over-current falling event input
#define CFG_COG_1_FE_MODULATOR  0x02    //Comparator 1 is duty cycle modulator falling event input
#define CFG_COG_1_FE_MAXDC      0X02    //PWM5 enabled as falling event input
#define CFG_COG_1_FM_OVRCRNT    0x00    //Mode for comparator 2 is immediate
#define CFG_COG_1_FM_MODULATOR  0x00    //Mode for comparator 1 is immediate
#define CFG_COG_1_FM_MAXDC      0X02    //Mode for PWM5 is after phase delay
#define CFG_COG_1_CON0          0x94    //Enable, HFINTOSC clock, half-bridge mode
#define CFG_COG_1_CON1          0x40    //Use delay chain for falling event dead band timing
#define CFG_COG_1_FE_DEADBAND   0x28    //~200nS delay for low-side turn-on
#define CFG_COG_1_RE_DEADBAND   0x00    //No rising dead band


#ifdef _USE_PRG2_FE_CLC1
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Configure CLC1 as AND gate to drive falling event trigger for PRG2:  lesser of 
 * COG high-side (A) output or PWM6 (slave).
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_CLC_1_DS_0      0x14    //PWM6 selected for d1T/g1
#define CFG_CLC_1_DS_1      0x14    //PWM6 selected for d2T/g2
#define CFG_CLC_1_DS_2      0x0B    //COG1 A out selected for d3T/g3
#define CFG_CLC_1_DS_3      0x0B    //COG1 A out selected for d4T/g4
#define CFG_CLC_1_LS_0      0x02    //d1T is gated into g1
#define CFG_CLC_1_LS_1      0x08    //d2T is gated into g2
#define CFG_CLC_1_LS_2      0x20    //d3T is gated into g3
#define CFG_CLC_1_LS_3      0x80    //d4T is gated into g4
#define CFG_CLC_1_POL       0x00    //No inverted polarities
#define CFG_CLC_1_CON       0x02    //Not enabled, no interrupts, AND cell mode
#endif

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Configure CLC2 as AND gate to drive low side output:  lesser of COG low-side
 * (B) output or PRG2 ramp threshold termination.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_CLC_2_DS_0      0x09    //CMP3 out selected for d1T/g1
#define CFG_CLC_2_DS_1      0x09    //CMP3 out selected for d2T/g2
#define CFG_CLC_2_DS_2      0x0C    //COG1 B out selected for d3T/g3
#define CFG_CLC_2_DS_3      0x0C    //COG1 B out selected for d4T/g4
#define CFG_CLC_2_LS_0      0x02    //d1T is gated into g1
#define CFG_CLC_2_LS_1      0x08    //d2T is gated into g2
#define CFG_CLC_2_LS_2      0x20    //d3T is gated into g3
#define CFG_CLC_2_LS_3      0x80    //d4T is gated into g4
#define CFG_CLC_2_POL       0x00    //No inverted polarities
#define CFG_CLC_2_CON       0x02    //Not enabled, no interrupts, AND cell mode


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Configure PRG1 as rising ramp source for modulator core
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_PRG_1_C0    0xBA            //Enabled, edge triggers, rising ramp, one-shot, no-go
#define CFG_PRG_1_C1    0x02            //Active high for rise & fall polarity select
#define CFG_PRG_1_VS    0x05            //Use DAC3 for voltage source
#define CFG_PRG_1_RS    0x07            //PWM5 is rising source trigger
#define CFG_PRG_1_FS    0x07            //PWM5 is falling source trigger

#ifdef _OUTPUT_12V
#define CFG_PRG_1_C2    0x00            //2uA current=0.2V/uS
#else
#define CFG_PRG_1_C2    0x0B            //7.5uA current=0.75V/uS
#endif


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Configure PRG2 as rising / falling ramp source to time maximum low-side on-time
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define CFG_PRG_2_C0    0x96            //Enabled, edge triggers, rising / falling ramp, one-shot, no-go
#define CFG_PRG_2_C1    0x02            //Fall event=active low, rise=active high
#define CFG_PRG_2_VS    0x05            //Use DAC3 for voltage source
#define CFG_PRG_2_C2    0x06            //5uA current=0.5V/uS
#define CFG_PRG_2_RS    0x07            //PWM5 is rising source trigger
#ifdef _USE_PRG2_FE_CLC1
#define CFG_PRG_2_FS    0x04            //CLC1 via PPS pin is falling source trigger
#else
#define CFG_PRG_2_FS    0x08            //PWM6 is falling source trigger
#endif


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                          Object definitions
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//Object to store a measured voltage with 3-digit mV resolution
typedef struct {
    unsigned int vmv;                   //Volts in 10K/1K places, mV in 100/10/1
    unsigned int volt;                  //Volts as integer
    unsigned int mv;                    //mV as integer 
} volts;

//Configuration record to control operation
//To calculate DC% from measured DC:  Config.odcp = ((Config.oldc * 100) / (CFG_PWM_5_FRQ + 1));
typedef struct {
    unsigned char sts;                  //System status code; see status codes
    unsigned char ochs;                 //Count of over-current channel A events
    unsigned char cogt;                 //Calculated COG on-time derived from function getCOGtime
    unsigned int  cogc;                 //Timer count of COG duty cycle
    unsigned int  odcp;                 //Open loop duty cycle % entered by user
    unsigned int  oldc;                 //Calculated 16-bit open loop duty cycle
    volts vdd;                          //Last measured Vdd
    volts bus;                          //Last measured bus volts
    volts out;                          //Last measured output volts
} opcf;


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                             Status codes
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#define STS_BOOT        0x00            //Startup
#define STS_RUN         0x01            //Running
#define STS_BUS_UVLO    0x02            //Stopped - bus under-volt lockout
#define STS_CHL_OCLO    0x03            //Stopped - over-current lockout
#define STS_OUT_UVLO    0x04            //Stopped - output under-volt lockout
#define STS_OUT_OVLO    0x05            //Stopped - output over-volt lockout
#define STS_BUS_OVLO    0x06            //Disabled - bus over-volt lockout

#define STS_CODE_ERR    0x02            //Codes < this value are not errors



/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                          Configure system clock
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//Bit 7=0       4x PLL is disabled
//Bit 6-3=1111  16 MHz HF
//Bit 2=0       n/a
//Bit 1-0=00    Clock determined by FOSC<2:0> in the Configuration Words
#define CFG_OSC_C   0x78        //16MHz, FOSC config word determines clock
#define _XTAL_FREQ  16000000    //Required to use __delayxxx macros; must match Fosc


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 *                      Configuration Bit Settings
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
// CONFIG1
#pragma config FOSC = INTOSC    // Oscillator Selection Bits (INTOSC oscillator: I/O function on CLKIN pin)
#pragma config WDTE = OFF       // Watchdog Timer Enable (WDT disabled)
#pragma config PWRTE = OFF      // Power-up Timer Enable (PWRT disabled)
#pragma config MCLRE = OFF      // MCLR Pin Function Select (MCLR/VPP pin function is digital input)
#pragma config CP = OFF         // Flash Program Memory Code Protection (Program memory code protection is disabled)
#pragma config BOREN = ON       // Brown-out Reset Enable (Brown-out Reset enabled)
#pragma config CLKOUTEN = OFF   // Clock Out Enable (CLKOUT function is disabled. I/O or oscillator function on the CLKOUT pin)
#pragma config IESO = OFF       // Internal/External Switchover Mode (Internal/External Switchover Mode is disabled)
#pragma config FCMEN = OFF      // Fail-Safe Clock Monitor Enable (Fail-Safe Clock Monitor is disabled)
// CONFIG2
#pragma config WRT = OFF        // Flash Memory Self-Write Protection (Write protection off)
#pragma config PPS1WAY = OFF    // Peripheral Pin Select one-way control (The PPSLOCK bit can be set and cleared repeatedly by software)
#pragma config ZCD = OFF        // Zero-cross detect disable (Zero-cross detect circuit is disabled at POR)
#pragma config PLLEN = OFF      // Phase Lock Loop enable (4x PLL is enabled when software sets the SPLLEN bit)
#pragma config STVREN = ON      // Stack Overflow/Underflow Reset Enable (Stack Overflow or Underflow will cause a Reset)
#pragma config BORV = LO        // Brown-out Reset Voltage Selection (Brown-out Reset Voltage (Vbor), low trip point selected.)
#pragma config LPBOR = OFF      // Low-Power Brown Out Reset (Low-Power BOR is disabled)
#pragma config LVP = OFF        // Low-Voltage Programming Enable (Low-voltage programming disabled)


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * Function declarations.  See source file for explanation of each function.
 *++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void interrupt InterruptExec(void);
void Initialize(void);
void main(void);
void ReadVoltage(unsigned char, unsigned char, volts *);
void chgDutyCycle(unsigned int, unsigned int);
void blinkErrorCode(unsigned char);


//End of file